@@ -15,6 +15,10 @@ from Orange.OrangeWidgets.OWGUI import LinkStyledItemDelegate, LinkRole
 from Orange.OrangeWidgets.OWGUI import BarItemDelegate
 from Orange.OrangeWidgets.OWWidget import *
 
+from Orange.OrangeWidgets.OWConcurrent import ThreadExecutor, Task
+
+from .utils.download import EnsureDownloaded
+
 from .. import obiGene, obiGeneSets, obiProb, obiTaxonomy
 
 NAME = "Gene Set Enrichment"
@@ -126,10 +130,6 @@ class OWSetEnrichment(OWWidget):
 
         self.loadSettings()
 
-        if self.signalManager:
-            self.signalManager.freeze(self).push()
-        QTimer.singleShot(50, self.updateHierarchy)
-
         box = OWGUI.widgetBox(self.controlArea, "Info")
         self.infoBox = OWGUI.widgetLabel(box, "Info")
         self.infoBox.setText("No data on input")
@@ -247,13 +247,21 @@ class OWSetEnrichment(OWWidget):
         self.updatingAnnotationsFlag = False
         self.currentAnnotatedCategories = []
 
+        self.setBlocking(True)
+
+        task = EnsureDownloaded(
+            [("Taxonomy", "ncbi_taxonomy.tar.gz"),
+             (obiGeneSets.sfdomain, "index.pck")]
+        )
+        task.finished.connect(self.updateHierarchy)
+
+        self._executor = ThreadExecutor()
+        self._executor.submit(task)
+
     def updateHierarchy(self):
         try:
-            self.progressBarInit()
-            with orngServerFiles.DownloadProgress.setredirect(self.progressBarSet):
-                all, local = obiGeneSets.list_all(), obiGeneSets.list_local()
-                organisms = set(obiTaxonomy.essential_taxids() + filter(None, [t[1] for t in all]))
-            self.progressBarFinished()
+            all, local = obiGeneSets.list_all(), obiGeneSets.list_local()
+            organisms = set(obiTaxonomy.essential_taxids() + filter(None, [t[1] for t in all]))
 
             organism_names = map(name_or_none, organisms)
             organisms = [taxid for taxid, name in zip(organisms, organism_names) \
@@ -264,8 +272,7 @@ class OWSetEnrichment(OWWidget):
             self.speciesComboBox.addItems([obiTaxonomy.name(id) for id in self.taxid_list])
             self.genesets = all
         finally:
-            if self.signalManager:
-                self.signalManager.freeze(self).pop() #setFreeze(self.signalManager.freezing - 1)
+            self.setBlocking(False)
 
     def setData(self, data=None):
         self.data = data
